@@ -22,14 +22,12 @@
 #include <config.h>
 
 #include <glib/gi18n.h>
-#include <gconf/gconf-client.h>
-#include "eggcellrendererkeys.h"
 #include "keyboard-shortcuts.h"
 #include "cc-keyboard-item.h"
 #include "wm-common.h"
 
-#define MAX_CUSTOM_SHORTCUTS 1000
-#define GCONF_BINDING_DIR "/desktop/gnome/keybindings"
+#define BINDINGS_SCHEMA "org.gnome.settings-daemon.plugins.media-keys"
+#define CUSTOM_KEYS_BASENAME "/org/gnome/settings-daemon/plugins/media-keys/custom-keybindings"
 #define WID(builder, name) (GTK_WIDGET (gtk_builder_get_object (builder, name)))
 
 typedef struct {
@@ -46,23 +44,13 @@ typedef struct {
   GArray *entries;
 } KeyList;
 
-typedef enum {
-  COMPARISON_NONE = 0,
-  COMPARISON_GT,
-  COMPARISON_LT,
-  COMPARISON_EQ
-} Comparison;
-
 typedef struct
 {
   CcKeyboardItemType type;
   char *schema; /* GSettings schema name, if any */
   char *description; /* description for GSettings types */
-  char *gettext_package; /* used for GConf type */
-  char *name; /* GConf key, GConf directory, or GSettings key name depending on type */
-  int value; /* Value for comparison */
-  char *key; /* GConf key name for the comparison */
-  Comparison comparison;
+  char *gettext_package;
+  char *name; /* GSettings schema path, or GSettings key name depending on type */
 } KeyListEntry;
 
 enum
@@ -81,13 +69,13 @@ enum
 
 static guint maybe_block_accels_id = 0;
 static gboolean block_accels = FALSE;
+static GSettings *settings = NULL;
 static GtkWidget *custom_shortcut_dialog = NULL;
 static GtkWidget *custom_shortcut_name_entry = NULL;
 static GtkWidget *custom_shortcut_command_entry = NULL;
 static GHashTable *kb_system_sections = NULL;
 static GHashTable *kb_apps_sections = NULL;
 static GHashTable *kb_user_sections = NULL;
-static guint workspace_num_notify_id = 0;
 
 static void
 free_key_array (GPtrArray *keys)
@@ -145,55 +133,19 @@ have_key_for_group (int group, const gchar *name)
         {
           CcKeyboardItem *item = g_ptr_array_index (keys, i);
 
-          if (g_strcmp0 (name, item->gconf_key) == 0)
-            return TRUE;
+	  if (item->type == CC_KEYBOARD_ITEM_TYPE_GSETTINGS &&
+	      g_strcmp0 (name, item->key) == 0)
+	    {
+	          return TRUE;
+	    }
+
+	  return FALSE;
         }
     }
 
   return FALSE;
 }
 
-static gboolean
-should_show_key (const KeyListEntry *entry)
-{
-  int value;
-  GConfClient *client;
-
-  if (entry->comparison == COMPARISON_NONE)
-    return TRUE;
-
-  g_return_val_if_fail (entry->key != NULL, FALSE);
-
-  /* FIXME: We'll need to change that when metacity/mutter
-   * uses GSettings */
-  g_assert (entry->type == CC_KEYBOARD_ITEM_TYPE_GCONF);
-
-  client = gconf_client_get_default();
-  value = gconf_client_get_int (client, entry->key, NULL);
-  g_object_unref (client);
-
-  switch (entry->comparison) {
-  case COMPARISON_NONE:
-    /* For compiler warnings */
-    g_assert_not_reached ();
-    return FALSE;
-  case COMPARISON_GT:
-    if (value > entry->value)
-      return TRUE;
-    break;
-  case COMPARISON_LT:
-    if (value < entry->value)
-      return TRUE;
-    break;
-  case COMPARISON_EQ:
-    if (value == entry->value)
-      return TRUE;
-    break;
-  }
-
-  return FALSE;
-}
-
 static gboolean
 keybinding_key_changed_foreach (GtkTreeModel   *model,
                                 GtkTreePath    *path,
@@ -223,6 +175,7 @@ item_changed (CcKeyboardItem *item,
   gtk_tree_model_foreach (item->model, (GtkTreeModelForeachFunc) keybinding_key_changed_foreach, item);
 }
 
+
 static void
 append_section (GtkBuilder         *builder,
                 const gchar        *title,
@@ -258,20 +211,14 @@ append_section (GtkBuilder         *builder,
       CcKeyboardItem *item;
       gboolean ret;
 
-      if (!should_show_key (&keys_list[i]))
-        continue;
-
-      if (have_key_for_group (group, keys_list[i].name)) /* FIXME broken for GSettings */
+      if (have_key_for_group (group, keys_list[i].name))
         continue;
 
       item = cc_keyboard_item_new (keys_list[i].type);
       switch (keys_list[i].type)
         {
-	case CC_KEYBOARD_ITEM_TYPE_GCONF:
-          ret = cc_keyboard_item_load_from_gconf (item, keys_list[i].gettext_package, keys_list[i].name);
-          break;
-	case CC_KEYBOARD_ITEM_TYPE_GCONF_DIR:
-          ret = cc_keyboard_item_load_from_gconf_dir (item, keys_list[i].name);
+	case CC_KEYBOARD_ITEM_TYPE_GSETTINGS_PATH:
+          ret = cc_keyboard_item_load_from_gsettings_path (item, keys_list[i].name, FALSE);
           break;
 	case CC_KEYBOARD_ITEM_TYPE_GSETTINGS:
 	  ret = cc_keyboard_item_load_from_gsettings (item,
@@ -335,9 +282,7 @@ parse_start_tag (GMarkupParseContext *ctx,
 {
   KeyList *keylist = (KeyList *) user_data;
   KeyListEntry key;
-  const char *name, *gconf_key, *schema, *description, *package;
-  int value;
-  Comparison comparison;
+  const char *name, *schema, *description, *package;
 
   name = NULL;
   schema = NULL;
@@ -416,9 +361,6 @@ parse_start_tag (GMarkupParseContext *ctx,
       || attr_values == NULL)
     return;
 
-  value = 0;
-  comparison = COMPARISON_NONE;
-  gconf_key = NULL;
   schema = NULL;
   description = NULL;
 
@@ -429,14 +371,6 @@ parse_start_tag (GMarkupParseContext *ctx,
           /* skip if empty */
           if (**attr_values)
             name = *attr_values;
-        } else if (g_str_equal (*attr_names, "value")) {
-          if (**attr_values) {
-            value = (int) g_ascii_strtoull (*attr_values, NULL, 0);
-          }
-        } else if (g_str_equal (*attr_names, "key")) {
-          if (**attr_values) {
-            gconf_key = *attr_values;
-          }
 	} else if (g_str_equal (*attr_names, "schema")) {
 	  if (**attr_values) {
 	   schema = *attr_values;
@@ -452,16 +386,6 @@ parse_start_tag (GMarkupParseContext *ctx,
 	        description = _(*attr_values);
 	      }
 	  }
-        } else if (g_str_equal (*attr_names, "comparison")) {
-          if (**attr_values) {
-            if (g_str_equal (*attr_values, "gt")) {
-              comparison = COMPARISON_GT;
-            } else if (g_str_equal (*attr_values, "lt")) {
-              comparison = COMPARISON_LT;
-            } else if (g_str_equal (*attr_values, "eq")) {
-              comparison = COMPARISON_EQ;
-            }
-          }
         }
 
       ++attr_names;
@@ -471,18 +395,17 @@ parse_start_tag (GMarkupParseContext *ctx,
   if (name == NULL)
     return;
 
+  if (schema == NULL &&
+      keylist->schema == NULL) {
+    g_debug ("Ignored GConf keyboard shortcut '%s'", name);
+    return;
+  }
+
   key.name = g_strdup (name);
-  if (schema != NULL ||
-      keylist->schema != NULL)
-    key.type = CC_KEYBOARD_ITEM_TYPE_GSETTINGS;
-  else
-    key.type = CC_KEYBOARD_ITEM_TYPE_GCONF;
-  key.value = value;
-  key.key = g_strdup (gconf_key);
+  key.type = CC_KEYBOARD_ITEM_TYPE_GSETTINGS;
   key.description = g_strdup (description);
   key.gettext_package = g_strdup (keylist->package);
   key.schema = schema ? g_strdup (schema) : g_strdup (keylist->schema);
-  key.comparison = comparison;
   g_array_append_val (keylist->entries, key);
 }
 
@@ -555,9 +478,6 @@ append_sections_from_file (GtkBuilder *builder, const gchar *path, const char *d
 
   /* Empty KeyListEntry to end the array */
   key.name = NULL;
-  key.key = NULL;
-  key.value = 0;
-  key.comparison = COMPARISON_NONE;
   g_array_append_val (keylist->entries, key);
 
   keys = (KeyListEntry *) g_array_free (keylist->entries, FALSE);
@@ -592,46 +512,35 @@ append_sections_from_file (GtkBuilder *builder, const gchar *path, const char *d
     g_free (entry->description);
     g_free (entry->gettext_package);
     g_free (entry->name);
-    g_free (entry->key);
   }
 
   g_free (keylist);
 }
 
 static void
-append_sections_from_gconf (GtkBuilder *builder, const gchar *gconf_path)
+append_sections_from_gsettings (GtkBuilder *builder)
 {
-  GConfClient *client;
-  GSList *custom_list, *l;
+  char **custom_paths;
   GArray *entries;
   KeyListEntry key;
+  int i;
 
-  /* load custom shortcuts from GConf */
+  /* load custom shortcuts from GSettings */
   entries = g_array_new (FALSE, TRUE, sizeof (KeyListEntry));
 
-  key.key = NULL;
-  key.value = 0;
-  key.comparison = COMPARISON_NONE;
-
-  client = gconf_client_get_default ();
-  custom_list = gconf_client_all_dirs (client, gconf_path, NULL);
-
-  for (l = custom_list; l != NULL; l = l->next)
+  custom_paths = g_settings_get_strv (settings, "custom-keybindings");
+  for (i = 0; custom_paths[i]; i++)
     {
-      key.name = g_strdup (l->data);
+      key.name = g_strdup (custom_paths[i]);
       if (!have_key_for_group (BINDING_GROUP_USER, key.name))
         {
-          key.type = CC_KEYBOARD_ITEM_TYPE_GCONF_DIR;
+          key.type = CC_KEYBOARD_ITEM_TYPE_GSETTINGS_PATH;
           g_array_append_val (entries, key);
         }
       else
         g_free (key.name);
-
-      g_free (l->data);
     }
-
-  g_slist_free (custom_list);
-  g_object_unref (client);
+  g_strfreev (custom_paths);
 
   if (entries->len > 0)
     {
@@ -662,7 +571,6 @@ reload_sections (GtkBuilder *builder)
 {
   gchar **wm_keybindings;
   GDir *dir;
-  const gchar *name;
   GtkTreeModel *sort_model;
   GtkTreeModel *section_model;
   GtkTreeModel *shortcut_model;
@@ -671,6 +579,7 @@ reload_sections (GtkBuilder *builder)
   GtkTreeView *section_treeview;
   GtkTreeSelection *selection;
   GtkTreeIter iter;
+  GHashTable *loaded_files;
 
   section_treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "section_treeview"));
   sort_model = gtk_tree_view_get_model (section_treeview);
@@ -706,10 +615,13 @@ reload_sections (GtkBuilder *builder)
   /* Load WM keybindings */
   wm_keybindings = wm_common_get_current_keybindings ();
 
+  loaded_files = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
   data_dirs = g_get_system_data_dirs ();
   for (i = 0; data_dirs[i] != NULL; i++)
     {
       char *dir_path;
+      const gchar *name;
 
       dir_path = g_build_filename (data_dirs[i], "gnome-control-center", "keybindings", NULL);
 
@@ -722,24 +634,31 @@ reload_sections (GtkBuilder *builder)
 
       for (name = g_dir_read_name (dir) ; name ; name = g_dir_read_name (dir))
         {
-	  if (g_str_has_suffix (name, ".xml"))
+          gchar *path;
+
+	  if (g_str_has_suffix (name, ".xml") == FALSE)
+	    continue;
+
+          if (g_hash_table_lookup (loaded_files, name) != NULL)
 	    {
-	      gchar *path;
-
-	      path = g_build_filename (dir_path, name, NULL);
-	      append_sections_from_file (builder, path, data_dirs[i], wm_keybindings);
-
-	      g_free (path);
+	      g_debug ("Not loading %s, it was already loaded from another directory", name);
+              continue;
 	    }
+
+	  g_hash_table_insert (loaded_files, g_strdup (name), GINT_TO_POINTER (1));
+	  path = g_build_filename (dir_path, name, NULL);
+	  append_sections_from_file (builder, path, data_dirs[i], wm_keybindings);
+	  g_free (path);
 	}
       g_free (dir_path);
       g_dir_close (dir);
     }
 
+  g_hash_table_destroy (loaded_files);
   g_strfreev (wm_keybindings);
 
   /* Load custom keybindings */
-  append_sections_from_gconf (builder, GCONF_BINDING_DIR);
+  append_sections_from_gsettings (builder);
 
   /* Select the first item */
   gtk_tree_model_get_iter_first (sort_model, &iter);
@@ -768,8 +687,8 @@ accel_set_func (GtkTreeViewColumn *tree_column,
     g_object_set (cell,
                   "visible", TRUE,
                   "editable", FALSE,
-                  "accel_key", item->keyval,
-                  "accel_mask", item->mask,
+                  "accel-key", item->keyval,
+                  "accel-mods", item->mask,
                   "keycode", item->keycode,
                   "style", PANGO_STYLE_ITALIC,
                   NULL);
@@ -777,8 +696,8 @@ accel_set_func (GtkTreeViewColumn *tree_column,
     g_object_set (cell,
                   "visible", TRUE,
                   "editable", TRUE,
-                  "accel_key", item->keyval,
-                  "accel_mask", item->mask,
+                  "accel-key", item->keyval,
+                  "accel-mods", item->mask,
                   "keycode", item->keycode,
                   "style", PANGO_STYLE_NORMAL,
                   NULL);
@@ -855,8 +774,6 @@ section_selection_changed (GtkTreeSelection *selection, gpointer data)
           return;
         }
 
-      gtk_widget_set_sensitive (WID (builder, "add-toolbutton"),
-                                group == BINDING_GROUP_USER);
       gtk_widget_set_sensitive (WID (builder, "remove-toolbutton"), FALSE);
 
       /* Fill the shortcut treeview with the keys for the selected section */
@@ -878,37 +795,52 @@ section_selection_changed (GtkTreeSelection *selection, gpointer data)
     }
 }
 
+typedef struct
+{
+  GtkTreeView *tree_view;
+  GtkTreePath *path;
+  GtkTreeViewColumn *column;
+} IdleData;
+
 static gboolean
 edit_custom_shortcut (CcKeyboardItem *item)
 {
   gint result;
-  const gchar *description, *command;
   gboolean ret;
+  GSettings *settings;
 
-  gtk_entry_set_text (GTK_ENTRY (custom_shortcut_name_entry), item->description ? item->description : "");
-  gtk_widget_set_sensitive (custom_shortcut_name_entry, item->desc_editable);
+  settings = g_settings_new_with_path (item->schema, item->gsettings_path);
+
+  g_settings_bind (settings, "name",
+                   G_OBJECT (custom_shortcut_name_entry), "text",
+                   G_SETTINGS_BIND_DEFAULT);
   gtk_widget_grab_focus (custom_shortcut_name_entry);
-  gtk_entry_set_text (GTK_ENTRY (custom_shortcut_command_entry), item->command ? item->command : "");
-  gtk_widget_set_sensitive (custom_shortcut_command_entry, item->cmd_editable);
+
+  g_settings_bind (settings, "command",
+                   G_OBJECT (custom_shortcut_command_entry), "text",
+                   G_SETTINGS_BIND_DEFAULT);
+
+  g_settings_delay (settings);
 
   gtk_window_present (GTK_WINDOW (custom_shortcut_dialog));
   result = gtk_dialog_run (GTK_DIALOG (custom_shortcut_dialog));
   switch (result)
     {
     case GTK_RESPONSE_OK:
-      description = gtk_entry_get_text (GTK_ENTRY (custom_shortcut_name_entry));
-      command = gtk_entry_get_text (GTK_ENTRY (custom_shortcut_command_entry));
-      g_object_set (G_OBJECT (item),
-		    "description", description,
-		    "command", command,
-		    NULL);
+      g_settings_apply (settings);
       ret = TRUE;
       break;
     default:
+      g_settings_revert (settings);
       ret = FALSE;
       break;
     }
 
+  g_settings_unbind (G_OBJECT (custom_shortcut_name_entry), "text");
+  g_settings_unbind (G_OBJECT (custom_shortcut_command_entry), "text");
+
+  g_object_unref (settings);
+
   gtk_widget_hide (custom_shortcut_dialog);
 
   return ret;
@@ -917,37 +849,36 @@ edit_custom_shortcut (CcKeyboardItem *item)
 static gboolean
 remove_custom_shortcut (GtkTreeModel *model, GtkTreeIter *iter)
 {
-  GConfClient *client;
-  gchar *base;
   CcKeyboardItem *item;
   GPtrArray *keys_array;
-  GError *err = NULL;
+  GVariantBuilder builder;
+  char **settings_paths;
+  int i;
 
   gtk_tree_model_get (model, iter,
                       DETAIL_KEYENTRY_COLUMN, &item,
                       -1);
 
   /* not a custom shortcut */
-  g_assert (item->type == CC_KEYBOARD_ITEM_TYPE_GCONF_DIR);
+  g_assert (item->type == CC_KEYBOARD_ITEM_TYPE_GSETTINGS_PATH);
 
-  client = gconf_client_get_default ();
+  g_settings_delay (item->settings);
+  g_settings_reset (item->settings, "name");
+  g_settings_reset (item->settings, "command");
+  g_settings_reset (item->settings, "binding");
+  g_settings_apply (item->settings);
+  g_settings_sync ();
 
-  base = g_strdup (item->gconf_key_dir);
+  settings_paths = g_settings_get_strv (settings, "custom-keybindings");
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
+  for (i = 0; settings_paths[i]; i++)
+    if (strcmp (settings_paths[i], item->gsettings_path) != 0)
+      g_variant_builder_add (&builder, "s", settings_paths[i]);
+  g_settings_set_value (settings,
+                        "custom-keybindings", g_variant_builder_end (&builder));
+  g_strfreev (settings_paths);
   g_object_unref (item);
 
-  if (gconf_client_recursive_unset (client, base, 0, &err) == FALSE)
-    {
-      g_warning ("Failed to unset GConf directory '%s': %s", base, err->message);
-      g_error_free (err);
-    }
-
-  g_free (base);
-  /* suggest sync now so the unset directory actually gets dropped;
-   * if we don't do this we may end up with 'zombie' shortcuts when
-   * restarting the app */
-  gconf_client_suggest_sync (client, NULL);
-  g_object_unref (client);
-
   keys_array = g_hash_table_lookup (get_hash_for_group (BINDING_GROUP_USER), _("Custom Shortcuts"));
   g_ptr_array_remove (keys_array, item);
 
@@ -965,7 +896,7 @@ update_custom_shortcut (GtkTreeModel *model, GtkTreeIter *iter)
                       DETAIL_KEYENTRY_COLUMN, &item,
                       -1);
 
-  g_assert (item->type == CC_KEYBOARD_ITEM_TYPE_GCONF_DIR);
+  g_assert (item->type == CC_KEYBOARD_ITEM_TYPE_GSETTINGS_PATH);
 
   edit_custom_shortcut (item);
   if (item->command == NULL || item->command[0] == '\0')
@@ -974,20 +905,24 @@ update_custom_shortcut (GtkTreeModel *model, GtkTreeIter *iter)
     }
   else
     {
-      GConfClient *client;
-
       gtk_list_store_set (GTK_LIST_STORE (model), iter,
                           DETAIL_KEYENTRY_COLUMN, item, -1);
-      client = gconf_client_get_default ();
-      if (item->description != NULL)
-        gconf_client_set_string (client, item->desc_gconf_key, item->description, NULL);
-      else
-        gconf_client_unset (client, item->desc_gconf_key, NULL);
-      gconf_client_set_string (client, item->cmd_gconf_key, item->command, NULL);
-      g_object_unref (client);
     }
 }
 
+static gboolean
+real_start_editing_cb (IdleData *idle_data)
+{
+  gtk_widget_grab_focus (GTK_WIDGET (idle_data->tree_view));
+  gtk_tree_view_set_cursor (idle_data->tree_view,
+                            idle_data->path,
+                            idle_data->column,
+                            TRUE);
+  gtk_tree_path_free (idle_data->path);
+  g_free (idle_data);
+  return FALSE;
+}
+
 static gboolean
 start_editing_cb (GtkTreeView    *tree_view,
                   GdkEventButton *event,
@@ -1005,10 +940,17 @@ start_editing_cb (GtkTreeView    *tree_view,
                                      &path, &column,
                                      NULL, NULL))
     {
+      IdleData *idle_data;
       GtkTreeModel *model;
       GtkTreeIter iter;
       CcKeyboardItem *item;
 
+      if (gtk_tree_path_get_depth (path) == 1)
+        {
+          gtk_tree_path_free (path);
+          return FALSE;
+        }
+
       model = gtk_tree_view_get_model (tree_view);
       gtk_tree_model_get_iter (model, &iter, path);
       gtk_tree_model_get (model, &iter,
@@ -1028,15 +970,15 @@ start_editing_cb (GtkTreeView    *tree_view,
         }
       else
         {
-          gtk_widget_grab_focus (GTK_WIDGET (tree_view));
-          gtk_tree_view_set_cursor (tree_view,
-                                    path,
-                                    item->desc_editable ? column :
-                                    gtk_tree_view_get_column (tree_view, 1),
-                                    TRUE);
+          idle_data = g_new (IdleData, 1);
+          idle_data->tree_view = tree_view;
+          idle_data->path = path;
+          idle_data->column = item->desc_editable ? column :
+                              gtk_tree_view_get_column (tree_view, 1);
+          g_idle_add ((GSourceFunc) real_start_editing_cb, idle_data);
+          block_accels = TRUE;
         }
       g_signal_stop_emission_by_name (tree_view, "button_press_event");
-      gtk_tree_path_free (path);
     }
   return TRUE;
 }
@@ -1057,6 +999,16 @@ start_editing_kb_cb (GtkTreeView *treeview,
                       DETAIL_KEYENTRY_COLUMN, &item,
                       -1);
 
+  if (item == NULL)
+    {
+      /* This is a section heading - expand or collapse */
+      if (gtk_tree_view_row_expanded (treeview, path))
+        gtk_tree_view_collapse_row (treeview, path);
+      else
+        gtk_tree_view_expand_row (treeview, path, FALSE);
+      return;
+    }
+
   /* if only the accel can be edited on the selected row
    * always select the accel column */
   if (item->desc_editable &&
@@ -1078,40 +1030,6 @@ start_editing_kb_cb (GtkTreeView *treeview,
     }
 }
 
-static void
-description_edited_callback (GtkCellRendererText *renderer,
-                             gchar               *path_string,
-                             gchar               *new_text,
-                             gpointer             user_data)
-{
-  GConfClient *client;
-  GtkTreeView *view = GTK_TREE_VIEW (user_data);
-  GtkTreeModel *model;
-  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
-  GtkTreeIter iter;
-  CcKeyboardItem *item;
-
-  model = gtk_tree_view_get_model (view);
-  gtk_tree_model_get_iter (model, &iter, path);
-  gtk_tree_path_free (path);
-
-  gtk_tree_model_get (model, &iter,
-                      DETAIL_KEYENTRY_COLUMN, &item,
-                      -1);
-
-  /* sanity check */
-  if (item == NULL || item->desc_gconf_key == NULL)
-    return;
-
-  g_assert (item->type == CC_KEYBOARD_ITEM_TYPE_GCONF_DIR);
-
-  client = gconf_client_get_default ();
-  if (!gconf_client_set_string (client, item->desc_gconf_key, new_text, NULL))
-    item->desc_editable = FALSE;
-
-  g_object_unref (client);
-}
-
 static const guint forbidden_keyvals[] = {
   /* Navigation keys */
   GDK_KEY_Home,
@@ -1135,13 +1053,13 @@ static const guint forbidden_keyvals[] = {
 static char*
 binding_name (guint                   keyval,
               guint                   keycode,
-              EggVirtualModifierType  mask,
+              GdkModifierType         mask,
               gboolean                translate)
 {
   if (keyval != 0 || keycode != 0)
     return translate ?
-        egg_virtual_accelerator_label (keyval, keycode, mask) :
-        egg_virtual_accelerator_name (keyval, keycode, mask);
+        gtk_accelerator_get_label_with_keycode (NULL, keyval, keycode, mask) :
+        gtk_accelerator_name_with_keycode (NULL, keyval, keycode, mask);
   else
     return g_strdup (translate ? _("Disabled") : "");
 }
@@ -1159,29 +1077,11 @@ keyval_is_forbidden (guint keyval)
   return FALSE;
 }
 
-static void
-show_error (GtkWindow *parent,
-            GError *err)
-{
-  GtkWidget *dialog;
-
-  dialog = gtk_message_dialog_new (parent,
-                                   GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
-                                   GTK_MESSAGE_WARNING,
-                                   GTK_BUTTONS_OK,
-                                   _("Error saving the new shortcut"));
-
-  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                            "%s", err->message);
-  gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_destroy (dialog);
-}
-
 typedef struct {
   CcKeyboardItem *orig_item;
   CcKeyboardItem *conflict_item;
   guint new_keyval;
-  EggVirtualModifierType new_mask;
+  GdkModifierType new_mask;
   guint new_keycode;
 } CcUniquenessData;
 
@@ -1231,7 +1131,7 @@ static void
 accel_edited_callback (GtkCellRendererText   *cell,
                        const char            *path_string,
                        guint                  keyval,
-                       EggVirtualModifierType mask,
+                       GdkModifierType        mask,
                        guint                  keycode,
                        GtkTreeView           *view)
 {
@@ -1256,7 +1156,7 @@ accel_edited_callback (GtkCellRendererText   *cell,
     return;
 
   /* CapsLock isn't supported as a keybinding modifier, so keep it from confusing us */
-  mask &= ~EGG_VIRTUAL_LOCK_MASK;
+  mask &= ~GDK_LOCK_MASK;
 
   data.orig_item = item;
   data.new_keyval = keyval;
@@ -1278,7 +1178,7 @@ accel_edited_callback (GtkCellRendererText   *cell,
     }
 
   /* Check for unmodified keys */
-  if ((mask == 0 || mask == GDK_SHIFT_MASK) && keycode != 0)
+  if (mask == 0 && keycode != 0)
     {
       if ((keyval >= GDK_KEY_a && keyval <= GDK_KEY_z)
            || (keyval >= GDK_KEY_A && keyval <= GDK_KEY_Z)
@@ -1311,9 +1211,11 @@ accel_edited_callback (GtkCellRendererText   *cell,
         gtk_widget_destroy (dialog);
 
         /* set it back to its previous value. */
-        egg_cell_renderer_keys_set_accelerator
-          (EGG_CELL_RENDERER_KEYS (cell),
-           item->keyval, item->keycode, item->mask);
+        g_object_set (G_OBJECT (cell),
+                      "accel-key", item->keyval,
+                      "keycode", item->keycode,
+                      "accel-mods", item->mask,
+                      NULL);
         return;
       }
     }
@@ -1364,10 +1266,11 @@ accel_edited_callback (GtkCellRendererText   *cell,
       else
         {
           /* set it back to its previous value. */
-          egg_cell_renderer_keys_set_accelerator (EGG_CELL_RENDERER_KEYS (cell),
-                                                  item->keyval,
-                                                  item->keycode,
-                                                  item->mask);
+        g_object_set (G_OBJECT (cell),
+                      "accel-key", item->keyval,
+                      "keycode", item->keycode,
+                      "accel-mods", item->mask,
+                      NULL);
         }
 
       return;
@@ -1407,15 +1310,6 @@ accel_cleared_callback (GtkCellRendererText *cell,
   g_object_set (G_OBJECT (item), "binding", "", NULL);
 }
 
-static void
-key_entry_controlling_key_changed (GConfClient *client,
-                                   guint        cnxn_id,
-                                   GConfEntry  *entry,
-                                   gpointer     user_data)
-{
-  reload_sections (user_data);
-}
-
 /* this handler is used to keep accels from activating while the user
  * is assigning a new shortcut so that he won't accidentally trigger one
  * of the widgets */
@@ -1432,33 +1326,29 @@ maybe_block_accels (GtkWidget *widget,
 }
 
 static gchar *
-find_free_gconf_key (GError **error)
+find_free_settings_path ()
 {
-  GConfClient *client;
+  char **used_names;
+  char *dir = NULL;
+  int i, num, n_names;
 
-  gchar *dir;
-  int i;
+  used_names = g_settings_get_strv (settings, "custom-keybindings");
+  n_names = g_strv_length (used_names);
 
-  client = gconf_client_get_default ();
-
-  for (i = 0; i < MAX_CUSTOM_SHORTCUTS; i++)
+  for (num = 0; dir == NULL; num++)
     {
-      dir = g_strdup_printf ("%s/custom%d", GCONF_BINDING_DIR, i);
-      if (!gconf_client_dir_exists (client, dir, NULL))
-        break;
-      g_free (dir);
-    }
+      char *tmp;
+      gboolean found = FALSE;
 
-  if (i == MAX_CUSTOM_SHORTCUTS)
-    {
-      dir = NULL;
-      g_set_error_literal (error,
-                           g_quark_from_string ("Keyboard Shortcuts"),
-                           0,
-                           _("Too many custom shortcuts"));
-    }
+      tmp = g_strdup_printf ("%s/custom%d/", CUSTOM_KEYS_BASENAME, num);
+      for (i = 0; i < n_names && !found; i++)
+        found = strcmp (used_names[i], tmp) == 0;
 
-  g_object_unref (client);
+      if (!found)
+        dir = tmp;
+      else
+        g_free (tmp);
+    }
 
   return dir;
 }
@@ -1469,31 +1359,15 @@ add_custom_shortcut (GtkTreeView  *tree_view,
 {
   CcKeyboardItem *item;
   GtkTreePath *path;
-  gchar *dir;
-  GError *error;
+  gchar *settings_path;
 
-  error = NULL;
-  dir = find_free_gconf_key (&error);
-  if (dir == NULL)
-    {
-      show_error (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tree_view))), error);
+  item = cc_keyboard_item_new (CC_KEYBOARD_ITEM_TYPE_GSETTINGS_PATH);
 
-      g_error_free (error);
-      return;
-    }
+  settings_path = find_free_settings_path ();
+  cc_keyboard_item_load_from_gsettings_path (item, settings_path, TRUE);
+  g_free (settings_path);
 
-  /* FIXME this is way ugly */
-  item = cc_keyboard_item_new (CC_KEYBOARD_ITEM_TYPE_GCONF_DIR);
-  item->gconf_key_dir = g_strdup (dir);
-  item->gconf_key = g_strconcat (dir, "/binding", NULL);
-  item->editable = TRUE;
   item->model = model;
-  item->desc_gconf_key = g_strconcat (dir, "/name", NULL);
-  item->description = g_strdup ("");
-  item->desc_editable = TRUE;
-  item->cmd_gconf_key = g_strconcat (dir, "/action", NULL);
-  item->command = g_strdup ("");
-  item->cmd_editable = TRUE;
 
   if (edit_custom_shortcut (item) &&
       item->command && item->command[0])
@@ -1501,19 +1375,9 @@ add_custom_shortcut (GtkTreeView  *tree_view,
       GPtrArray *keys_array;
       GtkTreeIter iter;
       GHashTable *hash;
-      GConfClient *client;
-
-      /* store in gconf */
-      client = gconf_client_get_default ();
-      gconf_client_set_string (client, item->gconf_key, "", NULL);
-      gconf_client_set_string (client, item->desc_gconf_key, item->description, NULL);
-      gconf_client_set_string (client, item->cmd_gconf_key, item->command, NULL);
-      g_object_unref (client);
-      g_object_unref (item);
-
-      /* Now setup the actual item we'll be adding */
-      item = cc_keyboard_item_new (CC_KEYBOARD_ITEM_TYPE_GCONF_DIR);
-      cc_keyboard_item_load_from_gconf_dir (item, dir);
+      GVariantBuilder builder;
+      char **settings_paths;
+      int i;
 
       hash = get_hash_for_group (BINDING_GROUP_USER);
       keys_array = g_hash_table_lookup (hash, _("Custom Shortcuts"));
@@ -1528,6 +1392,14 @@ add_custom_shortcut (GtkTreeView  *tree_view,
       gtk_list_store_append (GTK_LIST_STORE (model), &iter);
       gtk_list_store_set (GTK_LIST_STORE (model), &iter, DETAIL_KEYENTRY_COLUMN, item, -1);
 
+      settings_paths = g_settings_get_strv (settings, "custom-keybindings");
+      g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
+      for (i = 0; settings_paths[i]; i++)
+        g_variant_builder_add (&builder, "s", settings_paths[i]);
+      g_variant_builder_add (&builder, "s", item->gsettings_path);
+      g_settings_set_value (settings, "custom-keybindings",
+                            g_variant_builder_end (&builder));
+
       /* make the new shortcut visible */
       path = gtk_tree_model_get_path (model, &iter);
       gtk_tree_view_expand_to_path (tree_view, path);
@@ -1546,11 +1418,43 @@ add_button_clicked (GtkWidget  *button,
 {
   GtkTreeView *treeview;
   GtkTreeModel *model;
+  GtkTreeModel *section_model;
+  GtkTreeIter iter;
+  gboolean found, cont;
 
   treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder,
                                                     "shortcut_treeview"));
   model = gtk_tree_view_get_model (treeview);
 
+  /* Select the Custom Shortcuts section
+   * before adding the shortcut itself */
+  section_model = gtk_tree_view_get_model (GTK_TREE_VIEW (WID (builder, "section_treeview")));
+  cont = gtk_tree_model_get_iter_first (section_model, &iter);
+  found = FALSE;
+  while (cont)
+    {
+      BindingGroupType group;
+
+      gtk_tree_model_get (section_model, &iter,
+                          SECTION_GROUP_COLUMN, &group,
+                          -1);
+
+      if (group == BINDING_GROUP_USER)
+        {
+          found = TRUE;
+          break;
+        }
+      cont = gtk_tree_model_iter_next (section_model, &iter);
+    }
+  if (found)
+    {
+      GtkTreeSelection *selection;
+
+      selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (WID (builder, "section_treeview")));
+      gtk_tree_selection_select_iter (selection, &iter);
+    }
+
+  /* And add the shortcut */
   add_custom_shortcut (treeview, model);
 }
 
@@ -1700,13 +1604,11 @@ sections_separator_func (GtkTreeModel *model,
 static void
 setup_dialog (CcPanel *panel, GtkBuilder *builder)
 {
-  GConfClient *client;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   GtkWidget *widget;
   GtkTreeView *treeview;
   GtkTreeSelection *selection;
-  GSList *allowed_keys;
   CcShell *shell;
   GtkListStore *model;
   GtkTreeModelSort *sort_model;
@@ -1761,7 +1663,7 @@ setup_dialog (CcPanel *panel, GtkBuilder *builder)
   treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder,
                                                     "shortcut_treeview"));
 
-  client = gconf_client_get_default ();
+  settings = g_settings_new (BINDINGS_SCHEMA);
 
   g_signal_connect (treeview, "button_press_event",
                     G_CALLBACK (start_editing_cb), builder);
@@ -1771,12 +1673,9 @@ setup_dialog (CcPanel *panel, GtkBuilder *builder)
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (G_OBJECT (renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 
-  g_signal_connect (renderer, "edited",
-                    G_CALLBACK (description_edited_callback),
-                    treeview);
-
   column = gtk_tree_view_column_new_with_attributes (_("Action"),
                                                      renderer,
+                                                     "text", DETAIL_DESCRIPTION_COLUMN,
                                                      NULL);
   gtk_tree_view_column_set_cell_data_func (column, renderer, description_set_func, NULL, NULL);
   gtk_tree_view_column_set_resizable (column, FALSE);
@@ -1785,8 +1684,8 @@ setup_dialog (CcPanel *panel, GtkBuilder *builder)
   gtk_tree_view_append_column (treeview, column);
   gtk_tree_view_column_set_sort_column_id (column, DETAIL_DESCRIPTION_COLUMN);
 
-  renderer = (GtkCellRenderer *) g_object_new (EGG_TYPE_CELL_RENDERER_KEYS,
-                                               "accel_mode", EGG_CELL_RENDERER_KEYS_MODE_X,
+  renderer = (GtkCellRenderer *) g_object_new (GTK_TYPE_CELL_RENDERER_ACCEL,
+                                               "accel-mode", GTK_CELL_RENDERER_ACCEL_MODE_OTHER,
                                                NULL);
 
   g_signal_connect (renderer, "accel_edited",
@@ -1805,13 +1704,6 @@ setup_dialog (CcPanel *panel, GtkBuilder *builder)
   gtk_tree_view_append_column (treeview, column);
   gtk_tree_view_column_set_sort_column_id (column, DETAIL_KEYENTRY_COLUMN);
 
-  gconf_client_add_dir (client, GCONF_BINDING_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-  gconf_client_add_dir (client, "/apps/metacity/general", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-  workspace_num_notify_id = gconf_client_notify_add (client,
-                                                     "/apps/metacity/general/num_workspaces",
-                                                     (GConfClientNotifyFunc) key_entry_controlling_key_changed,
-                                                     builder, NULL, NULL);
-
   model = gtk_list_store_new (DETAIL_N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER);
   gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
                                    DETAIL_KEYENTRY_COLUMN,
@@ -1839,21 +1731,6 @@ setup_dialog (CcPanel *panel, GtkBuilder *builder)
                     G_CALLBACK (shortcut_selection_changed),
                     WID (builder, "remove-toolbutton"));
 
-  /* FIXME those use GSettings now */
-  allowed_keys = gconf_client_get_list (client,
-                                        GCONF_BINDING_DIR "/allowed_keys",
-                                        GCONF_VALUE_STRING,
-                                        NULL);
-  if (allowed_keys != NULL)
-    {
-      g_slist_foreach (allowed_keys, (GFunc)g_free, NULL);
-      g_slist_free (allowed_keys);
-      gtk_widget_set_sensitive (WID (builder, "add-toolbutton"),
-                                FALSE);
-    }
-
-  g_object_unref (client);
-
   /* setup the custom shortcut dialog */
   custom_shortcut_dialog = WID (builder,
                                 "custom-shortcut-dialog");
@@ -1919,13 +1796,5 @@ keyboard_shortcuts_dispose (CcPanel *panel)
           g_hash_table_destroy (kb_user_sections);
           kb_user_sections = NULL;
         }
-      if (workspace_num_notify_id != 0)
-        {
-          GConfClient *client;
-          client = gconf_client_get_default ();
-          gconf_client_notify_remove (client, workspace_num_notify_id);
-          workspace_num_notify_id = 0;
-          g_object_unref (client);
-        }
     }
 }
