@@ -214,7 +214,8 @@ get_primary_device_cb (GObject *source_object, GAsyncResult *res, gpointer user_
   result = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object), res, &error);
   if (result == NULL)
     {
-      g_printerr ("Error getting primary device: %s\n", error->message);
+      if (g_strstr_len (error->message, -1, "no primary device") == NULL)
+        g_printerr ("Error getting primary device: %s\n", error->message);
       g_error_free (error);
       gtk_widget_hide (status_box);
       return;
@@ -399,7 +400,7 @@ combo_time_changed_cb (GtkWidget *widget, CcPowerPanel *self)
                       1, &value,
                       -1);
 
-  /* set both battery and ac keys */
+  /* set both keys */
   g_settings_set_int (self->priv->gsd_settings, key, value);
 }
 
@@ -541,14 +542,11 @@ set_ac_battery_ui_mode (CcPowerPanel *self)
   g_ptr_array_unref (devices);
 out:
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-                                               "vbox_actions"));
+                                               "box_header"));
   gtk_widget_set_visible (widget, has_batteries);
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-                                               "vbox_battery"));
+                                               "combobox_sleep_battery"));
   gtk_widget_set_visible (widget, has_batteries);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-                                               "hbox_ac"));
-  gtk_widget_set_visible (widget, !has_batteries);
 }
 
 static void
@@ -556,7 +554,6 @@ cc_power_panel_init (CcPowerPanel *self)
 {
   GError     *error;
   GtkWidget  *widget;
-  GtkWidget  *target;
   gint        value;
 
   self->priv = POWER_PANEL_PRIVATE (self);
@@ -599,70 +596,12 @@ cc_power_panel_init (CcPowerPanel *self)
                     G_CALLBACK (on_lock_settings_changed),
                     self);
 
-  /* setup the checkboxes correcty */
-  widget = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "checkbutton_sleep_ac"));
-  target = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "combobox_sleep_ac"));
-  g_object_bind_property (widget, "active",
-                          target, "sensitive",
-                          G_BINDING_SYNC_CREATE);
-  gtk_widget_set_sensitive (widget,
-                            up_client_get_can_suspend (self->priv->up_client));
-
-  widget = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "checkbutton_sleep_battery"));
-  target = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "combobox_sleep_battery"));
-  g_object_bind_property (widget, "active",
-                          target, "sensitive",
-                          G_BINDING_SYNC_CREATE);
-  gtk_widget_set_sensitive (widget,
-                            up_client_get_can_suspend (self->priv->up_client));
-
-  widget = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "checkbutton_sleep"));
-  target = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "combobox_sleep"));
-  g_object_bind_property (widget, "active",
-                          target, "sensitive",
-                          G_BINDING_SYNC_CREATE);
-  gtk_widget_set_sensitive (widget,
-                            up_client_get_can_suspend (self->priv->up_client));
-
-  /* bind the checkboxes */
-  widget = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "checkbutton_sleep_ac"));
-  g_settings_bind (self->priv->gsd_settings,
-                   "sleep-inactive-ac",
-                   widget, "active",
-                   G_SETTINGS_BIND_DEFAULT);
-  widget = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "checkbutton_sleep_battery"));
-  g_settings_bind (self->priv->gsd_settings,
-                   "sleep-inactive-battery",
-                   widget, "active",
-                   G_SETTINGS_BIND_DEFAULT);
-  widget = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "checkbutton_sleep"));
-  g_settings_bind (self->priv->gsd_settings,
-                   "sleep-inactive-ac",
-                   widget, "active",
-                   G_SETTINGS_BIND_DEFAULT);
-
   /* auto-sleep time */
   value = g_settings_get_int (self->priv->gsd_settings, "sleep-inactive-ac-timeout");
   widget = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
                                                "combobox_sleep_ac"));
   set_value_for_combo (GTK_COMBO_BOX (widget), value);
   g_object_set_data (G_OBJECT(widget), "_gsettings_key", "sleep-inactive-ac-timeout");
-  g_signal_connect (widget, "changed",
-                    G_CALLBACK (combo_time_changed_cb),
-                    self);
-  widget = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-                                               "combobox_sleep"));
-  set_value_for_combo (GTK_COMBO_BOX (widget), value);
-  g_object_set_data (G_OBJECT(widget), "_gsettings_key", "sleep-inactive-ac-timeout");
   g_signal_connect (widget, "changed",
                     G_CALLBACK (combo_time_changed_cb),
                     self);
