@@ -377,7 +377,6 @@ attach_to_cups_notifier (gpointer data)
       priv->subscription_renewal_id =
         g_timeout_add_seconds (RENEW_INTERVAL, renew_subscription, self);
 
-      error = NULL;
       priv->cups_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
                                                         0,
                                                         NULL,
@@ -387,9 +386,10 @@ attach_to_cups_notifier (gpointer data)
                                                         NULL,
                                                         &error);
 
-      if (error)
+      if (!priv->cups_proxy)
         {
           g_warning ("%s", error->message);
+          g_error_free (error);
           return;
         }
 
@@ -497,12 +497,14 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
   GtkTreeIter             iter;
   GtkWidget              *widget;
   gboolean                sensitive;
+  GValue                  value = G_VALUE_INIT;
   gchar                  *printer_make_and_model = NULL;
   gchar                  *printer_model = NULL;
   gchar                  *reason = NULL;
   gchar                 **printer_reasons = NULL;
   gchar                  *marker_types = NULL;
   gchar                  *printer_name = NULL;
+  gchar                  *printer_icon = NULL;
   gchar                  *printer_type = NULL;
   gchar                  *active_jobs = NULL;
   gchar                  *supply_type = NULL;
@@ -579,6 +581,7 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       gtk_tree_model_get (model, &iter,
 			  PRINTER_ID_COLUMN, &id,
 			  PRINTER_NAME_COLUMN, &printer_name,
+			  PRINTER_ICON_COLUMN, &printer_icon,
 			  -1);
     }
   else
@@ -595,8 +598,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
         g_strcmp0 (priv->dests[priv->current_dest].name,
                    priv->jobs[priv->current_job].dest) == 0))
     {
-      actualize_jobs_list (self);
-
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "job-release-button");
       gtk_widget_set_sensitive (widget, FALSE);
@@ -608,6 +609,8 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "job-cancel-button");
       gtk_widget_set_sensitive (widget, FALSE);
+
+      actualize_jobs_list (self);
     }
 
   actualize_allowed_users_list (self);
@@ -756,6 +759,19 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
             }
         }
 
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-icon");
+      g_value_init (&value, G_TYPE_INT);
+      g_object_get_property ((GObject *) widget, "icon-size", &value);
+
+      if (printer_icon)
+        {
+          gtk_image_set_from_icon_name ((GtkImage *) widget, printer_icon, g_value_get_int (&value));
+          g_free (printer_icon);
+        }
+      else
+        gtk_image_set_from_icon_name ((GtkImage *) widget, "printer", g_value_get_int (&value));
+
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-name-label");
 
@@ -1398,7 +1414,6 @@ job_selection_changed_cb (GtkTreeSelection *selection,
 static void
 populate_jobs_list (CcPrintersPanel *self)
 {
-
   CcPrintersPanelPrivate *priv;
   GtkTreeViewColumn      *column;
   GtkCellRenderer        *renderer;
@@ -1407,8 +1422,6 @@ populate_jobs_list (CcPrintersPanel *self)
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
-  actualize_jobs_list (self);
-
   treeview = (GtkTreeView*)
     gtk_builder_get_object (priv->builder, "job-treeview");
 
@@ -1438,6 +1451,8 @@ populate_jobs_list (CcPrintersPanel *self)
 
   g_signal_connect (gtk_tree_view_get_selection (treeview),
                     "changed", G_CALLBACK (job_selection_changed_cb), self);
+
+  actualize_jobs_list (self);
 }
 
 enum
@@ -1550,6 +1565,7 @@ job_process_cb (GtkButton *button,
   CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
   DBusGProxy             *proxy;
   GtkWidget              *widget;
+  gboolean                result = TRUE;
   GError                 *error = NULL;
   char                   *ret_error = NULL;
   int                     id = -1;
@@ -1574,45 +1590,46 @@ job_process_cb (GtkButton *button,
       if ((GtkButton*) gtk_builder_get_object (priv->builder,
                                                "job-cancel-button") ==
           button)
-        dbus_g_proxy_call (proxy, "JobCancelPurge", &error,
-                           G_TYPE_INT, id,
-                           G_TYPE_BOOLEAN, FALSE,
-                           G_TYPE_INVALID,
-                           G_TYPE_STRING, &ret_error,
-                           G_TYPE_INVALID);
+        result = dbus_g_proxy_call (proxy, "JobCancelPurge", &error,
+                                    G_TYPE_INT, id,
+                                    G_TYPE_BOOLEAN, FALSE,
+                                    G_TYPE_INVALID,
+                                    G_TYPE_STRING, &ret_error,
+                                    G_TYPE_INVALID);
       else if ((GtkButton*) gtk_builder_get_object (priv->builder,
                                                         "job-hold-button") ==
                button)
-        dbus_g_proxy_call (proxy, "JobSetHoldUntil", &error,
-                           G_TYPE_INT, id,
-                           G_TYPE_STRING, "indefinite",
-                           G_TYPE_INVALID,
-                           G_TYPE_STRING, &ret_error,
-                           G_TYPE_INVALID);
+        result = dbus_g_proxy_call (proxy, "JobSetHoldUntil", &error,
+                                    G_TYPE_INT, id,
+                                    G_TYPE_STRING, "indefinite",
+                                    G_TYPE_INVALID,
+                                    G_TYPE_STRING, &ret_error,
+                                    G_TYPE_INVALID);
       else if ((GtkButton*) gtk_builder_get_object (priv->builder,
                                                         "job-release-button") ==
                button)
-        dbus_g_proxy_call (proxy, "JobSetHoldUntil", &error,
-                           G_TYPE_INT, id,
-                           G_TYPE_STRING, "no-hold",
-                           G_TYPE_INVALID,
-                           G_TYPE_STRING, &ret_error,
-                           G_TYPE_INVALID);
+        result = dbus_g_proxy_call (proxy, "JobSetHoldUntil", &error,
+                                    G_TYPE_INT, id,
+                                    G_TYPE_STRING, "no-hold",
+                                    G_TYPE_INVALID,
+                                    G_TYPE_STRING, &ret_error,
+                                    G_TYPE_INVALID);
 
       g_object_unref (proxy);
 
-      if (error || (ret_error && ret_error[0] != '\0'))
+      if (!result || (ret_error && ret_error[0] != '\0'))
         {
-          if (error)
-            g_warning ("%s", error->message);
+          if (!result)
+            {
+              g_warning ("%s", error->message);
+              g_error_free (error);
+            }
 
           if (ret_error && ret_error[0] != '\0')
             g_warning ("%s", ret_error);
         }
       else
         actualize_jobs_list (self);
-
-      g_clear_error (&error);
   }
 
   widget = (GtkWidget*)
@@ -1738,7 +1755,7 @@ supply_levels_draw_cb (GtkWidget *widget,
           GdkRGBA   border_color = {0.0, 0.0, 0.0, 1.0};
           GSList   *markers = NULL;
           GSList   *tmp_list = NULL;
-          GValue    int_val = {0};
+          GValue    int_val = G_VALUE_INIT;
           gchar   **marker_levelsv = NULL;
           gchar   **marker_colorsv = NULL;
           gchar   **marker_namesv = NULL;
@@ -2173,7 +2190,7 @@ test_page_cb (GtkButton *button,
       gchar        *printer_uri = NULL;
       gchar        *filename = NULL;
       gchar        *resource = NULL;
-      ipp_t        *response;
+      ipp_t        *response = NULL;
       ipp_t        *request;
 
       if ((datadir = getenv ("CUPS_DATADIR")) != NULL)
@@ -2455,6 +2472,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
   http_t                 *http;
   gchar                  *objects[] = { "main-vbox", NULL };
   GtkStyleContext        *context;
+  guint                   builder_result;
 
   priv = self->priv = PRINTERS_PANEL_PRIVATE (self);
 
@@ -2486,11 +2504,11 @@ cc_printers_panel_init (CcPrintersPanel *self)
   priv->permission = NULL;
   priv->lockdown_settings = NULL;
 
-  gtk_builder_add_objects_from_file (priv->builder,
-                                     DATADIR"/printers.ui",
-                                     objects, &error);
+  builder_result = gtk_builder_add_objects_from_file (priv->builder,
+                                                      DATADIR"/printers.ui",
+                                                      objects, &error);
 
-  if (error)
+  if (builder_result == 0)
     {
       /* Translators: The XML file containing user interface can not be loaded */
       g_warning (_("Could not load ui: %s"), error->message);
