@@ -40,12 +40,13 @@
 
 #include "marshal.h"
 
+#include "shell/cc-editable-entry.h"
+
 #include "um-user.h"
 #include "um-user-manager.h"
 
 #include "um-strength-bar.h"
 #include "um-editable-button.h"
-#include "cc-editable-entry.h"
 #include "um-editable-combo.h"
 
 #include "um-account-dialog.h"
@@ -875,8 +876,8 @@ add_unlock_tooltip (GtkWidget *button)
         gchar *names[3];
         GIcon *icon;
 
-        names[0] = "changes-prevent-symbolic";
-        names[1] = "changes-prevent";
+        names[0] = "changes-allow-symbolic";
+        names[1] = "changes-allow";
         names[2] = NULL;
         icon = (GIcon *)g_themed_icon_new_from_names (names, -1);
         /* Translator comments:
@@ -922,18 +923,18 @@ on_permission_changed (GPermission *permission,
         widget = get_widget (d, "add-user-toolbutton");
         gtk_widget_set_sensitive (widget, is_authorized);
         if (is_authorized) {
-                setup_tooltip_with_embedded_icon (widget, _("Create a user"), NULL, NULL);
+                setup_tooltip_with_embedded_icon (widget, _("Create a user account"), NULL, NULL);
         }
         else {
                 gchar *names[3];
                 GIcon *icon;
 
-                names[0] = "changes-prevent-symbolic";
-                names[1] = "changes-prevent";
+                names[0] = "changes-allow-symbolic";
+                names[1] = "changes-allow";
                 names[2] = NULL;
                 icon = (GIcon *)g_themed_icon_new_from_names (names, -1);
                 setup_tooltip_with_embedded_icon (widget,
-                                                  _("To create a user,\nclick the * icon first"),
+                                                  _("To create a user account,\nclick the * icon first"),
                                                   "*",
                                                   icon);
                 g_object_unref (icon);
@@ -942,19 +943,19 @@ on_permission_changed (GPermission *permission,
         widget = get_widget (d, "remove-user-toolbutton");
         gtk_widget_set_sensitive (widget, is_authorized && !self_selected);
         if (is_authorized) {
-                setup_tooltip_with_embedded_icon (widget, _("Delete the selected user"), NULL, NULL);
+                setup_tooltip_with_embedded_icon (widget, _("Delete the selected user account"), NULL, NULL);
         }
         else {
                 gchar *names[3];
                 GIcon *icon;
 
-                names[0] = "changes-prevent-symbolic";
-                names[1] = "changes-prevent";
+                names[0] = "changes-allow-symbolic";
+                names[1] = "changes-allow";
                 names[2] = NULL;
                 icon = (GIcon *)g_themed_icon_new_from_names (names, -1);
 
                 setup_tooltip_with_embedded_icon (widget,
-                                                  _("To delete the selected user,\nclick the * icon first"),
+                                                  _("To delete the selected user account,\nclick the * icon first"),
                                                   "*",
                                                   icon);
                 g_object_unref (icon);
@@ -1157,7 +1158,7 @@ setup_main_window (UmUserPanelPrivate *d)
         gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (column), cell, "pixbuf", FACE_COL);
         gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (column), cell, "visible", USER_ROW_COL);
         cell = gtk_cell_renderer_text_new ();
-        g_object_set (cell, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+        g_object_set (cell, "ellipsize", PANGO_ELLIPSIZE_END, "width-chars", 18, NULL);
         gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (column), cell, TRUE);
         gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (column), cell, "markup", NAME_COL);
         gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (column), cell, "visible", USER_ROW_COL);
@@ -1165,17 +1166,19 @@ setup_main_window (UmUserPanelPrivate *d)
         gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (column), cell, TRUE);
         gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (column), cell, "markup", TITLE_COL);
         gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (column), cell, "visible", HEADING_ROW_COL);
+
+        gtk_tree_view_append_column (GTK_TREE_VIEW (userlist), column);
+
         cell = gtk_cell_renderer_pixbuf_new ();
+        column = gtk_tree_view_column_new ();
         gtk_tree_view_column_pack_start (column, cell, FALSE);
-        gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (column), cell, "visible", USER_ROW_COL);
+        gtk_tree_view_append_column (GTK_TREE_VIEW (userlist), column);
         gtk_tree_view_column_set_cell_data_func (column,
                                                  cell,
                                                  (GtkTreeCellDataFunc) autologin_cell_data_func,
                                                  d,
                                                  NULL);
 
-        gtk_tree_view_append_column (GTK_TREE_VIEW (userlist), column);
-
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (userlist));
         gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
         g_signal_connect (selection, "changed", G_CALLBACK (selected_user_changed), d);
@@ -1215,17 +1218,17 @@ setup_main_window (UmUserPanelPrivate *d)
         on_permission_changed (d->permission, NULL, d);
 
         button = get_widget (d, "add-user-toolbutton");
-        names[0] = "changes-prevent-symbolic";
-        names[1] = "changes-prevent";
+        names[0] = "changes-allow-symbolic";
+        names[1] = "changes-allow";
         names[2] = NULL;
         icon = (GIcon *)g_themed_icon_new_from_names (names, -1);
         setup_tooltip_with_embedded_icon (button,
-                                          _("To create a user,\nclick the * icon first"),
+                                          _("To create a user account,\nclick the * icon first"),
                                           "*",
                                           icon);
         button = get_widget (d, "remove-user-toolbutton");
         setup_tooltip_with_embedded_icon (button,
-                                          _("To delete the selected user,\nclick the * icon first"),
+                                          _("To delete the selected user account,\nclick the * icon first"),
                                           "*",
                                           icon);
         g_object_unref (icon);
