@@ -54,6 +54,8 @@ cc_wacom_nav_button_update (CcWacomNavButton *nav)
 {
 	CcWacomNavButtonPrivate *priv = nav->priv;
 	int num_pages;
+	int current_page;
+	char *text;
 
 	if (priv->notebook == NULL) {
 		gtk_widget_hide (GTK_WIDGET (nav));
@@ -71,26 +73,23 @@ cc_wacom_nav_button_update (CcWacomNavButton *nav)
 
 	g_assert (num_pages >= 1);
 
-	if (num_pages == 1) {
+	if (num_pages == 1)
 		gtk_widget_hide (GTK_WIDGET (nav));
-	} else {
-		int current_page;
-		char *text;
-
+	else
 		gtk_widget_show (GTK_WIDGET (nav));
-		current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
-		if (current_page < 0)
-			return;
-		if (priv->ignore_first_page)
-			current_page--;
-		gtk_widget_set_sensitive (priv->prev, current_page == 0 ? FALSE : TRUE);
-		gtk_widget_set_sensitive (priv->next, current_page + 1 == num_pages ? FALSE : TRUE);
 
-		text = g_strdup_printf (_("%d of %d"),
-					current_page + 1,
-					priv->ignore_first_page ? num_pages - 1 : num_pages);
-		gtk_label_set_text (GTK_LABEL (priv->label), text);
-	}
+	current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
+	if (current_page < 0)
+		return;
+	if (priv->ignore_first_page)
+		current_page--;
+	gtk_widget_set_sensitive (priv->prev, current_page == 0 ? FALSE : TRUE);
+	gtk_widget_set_sensitive (priv->next, current_page + 1 == num_pages ? FALSE : TRUE);
+
+	text = g_strdup_printf (_("%d of %d"),
+				current_page + 1,
+				num_pages);
+	gtk_label_set_text (GTK_LABEL (priv->label), text);
 }
 
 static void
@@ -211,15 +210,21 @@ cc_wacom_nav_button_init (CcWacomNavButton *self)
 {
 	CcWacomNavButtonPrivate *priv;
 	GtkStyleContext *context;
-	GtkWidget *image;
+	GtkWidget *image, *box;
 
 	priv = self->priv = WACOM_NAV_BUTTON_PRIVATE (self);
-	context = gtk_widget_get_style_context (GTK_WIDGET (self));
-	gtk_style_context_add_class (context, GTK_STYLE_CLASS_LINKED);
 
 	/* Label */
 	priv->label = gtk_label_new (NULL);
 	gtk_style_context_add_class (gtk_widget_get_style_context (priv->label), "dim-label");
+	gtk_box_pack_start (GTK_BOX (self), priv->label,
+			    FALSE, FALSE, 8);
+
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	context = gtk_widget_get_style_context (GTK_WIDGET (box));
+	gtk_style_context_add_class (context, GTK_STYLE_CLASS_LINKED);
+	gtk_box_pack_start (GTK_BOX (self), box,
+			    FALSE, FALSE, 8);
 
 	/* Prev button */
 	priv->prev = gtk_button_new ();
@@ -227,7 +232,7 @@ cc_wacom_nav_button_init (CcWacomNavButton *self)
 	gtk_container_add (GTK_CONTAINER (priv->prev), image);
 	g_signal_connect (G_OBJECT (priv->prev), "clicked",
 			  G_CALLBACK (prev_clicked), self);
-	gtk_widget_set_valign (priv->prev, GTK_ALIGN_START);
+	gtk_widget_set_valign (priv->prev, GTK_ALIGN_CENTER);
 
 	/* Next button */
 	priv->next = gtk_button_new ();
@@ -235,18 +240,15 @@ cc_wacom_nav_button_init (CcWacomNavButton *self)
 	gtk_container_add (GTK_CONTAINER (priv->next), image);
 	g_signal_connect (G_OBJECT (priv->next), "clicked",
 			  G_CALLBACK (next_clicked), self);
-	gtk_widget_set_valign (priv->next, GTK_ALIGN_START);
+	gtk_widget_set_valign (priv->next, GTK_ALIGN_CENTER);
 
-	gtk_box_pack_start (GTK_BOX (self), priv->label,
-			    FALSE, FALSE, 8);
-	gtk_box_pack_start (GTK_BOX (self), priv->prev,
+	gtk_box_pack_start (GTK_BOX (box), priv->prev,
 			    FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (self), priv->next,
+	gtk_box_pack_start (GTK_BOX (box), priv->next,
 			    FALSE, FALSE, 0);
 
 	gtk_widget_show (priv->label);
-	gtk_widget_show_all (priv->prev);
-	gtk_widget_show_all (priv->next);
+	gtk_widget_show_all (box);
 }
 
 GtkWidget *
