@@ -31,6 +31,7 @@
 #include <string.h>
 
 #define WID(x) (GtkWidget *) gtk_builder_get_object (priv->builder, x)
+#define CWID(x) (GtkContainer *) gtk_builder_get_object (priv->builder, x)
 
 G_DEFINE_TYPE (CcWacomPage, cc_wacom_page, GTK_TYPE_BOX)
 
@@ -42,6 +43,7 @@ G_DEFINE_TYPE (CcWacomPage, cc_wacom_page, GTK_TYPE_BOX)
 
 struct _CcWacomPagePrivate
 {
+	CcWacomPanel   *panel;
 	GsdWacomDevice *stylus, *eraser;
 	GtkBuilder     *builder;
 	GtkWidget      *nav;
@@ -126,7 +128,8 @@ finish_calibration (CalibArea *area,
 
 static gboolean
 run_calibration (CcWacomPage *page,
-		 gint        *cal)
+		 gint        *cal,
+		 gint         monitor)
 {
 	XYinfo old_axis;
 
@@ -138,7 +141,7 @@ run_calibration (CcWacomPage *page,
 	old_axis.y_max = cal[3];
 
 	page->priv->area = calib_area_new (NULL,
-					   0, /* FIXME */
+					   monitor,
 					   finish_calibration,
 					   page,
 					   &old_axis,
@@ -156,6 +159,16 @@ calibrate_button_clicked_cb (GtkButton   *button,
 	GVariant *variant;
 	int *current;
 	gsize ncal;
+	gint monitor;
+
+	monitor = gsd_wacom_device_get_display_monitor (page->priv->stylus);
+	if (monitor < 0) {
+		/* The display the tablet should be mapped to could not be located.
+		 * This shouldn't happen if the EDID data is good...
+		 */
+		g_critical("Output associated with the tablet is not connected. Unable to calibrate.");
+		return;
+	}
 
 	variant = g_settings_get_value (page->priv->wacom_settings, "area");
 	current = (int *) g_variant_get_fixed_array (variant, &ncal, sizeof (gint32));
@@ -180,7 +193,7 @@ calibrate_button_clicked_cb (GtkButton   *button,
 		g_free (device_cal);
 	}
 
-	run_calibration (page, calibration);
+	run_calibration (page, calibration, monitor);
 	gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 }
 
@@ -250,6 +263,14 @@ combobox_text_cellrenderer (GtkComboBox *combo, int name_column)
 					"text", BUTTONNAME_COLUMN, NULL);
 }
 
+static gboolean
+display_clicked_cb (GtkButton   *button,
+		    CcWacomPage *page)
+{
+	cc_wacom_panel_switch_to_panel (page->priv->panel, "display");
+	return TRUE;
+}
+
 /* Boilerplate code goes below */
 
 static void
@@ -359,6 +380,9 @@ cc_wacom_page_init (CcWacomPage *self)
 	g_signal_connect (G_OBJECT (sw), "notify::active",
 			  G_CALLBACK (left_handed_toggled_cb), self);
 
+	g_signal_connect (G_OBJECT (WID ("display-link")), "activate-link",
+			  G_CALLBACK (display_clicked_cb), self);
+
 	priv->nav = cc_wacom_nav_button_new ();
 	gtk_grid_attach (GTK_GRID (box), priv->nav, 0, 0, 1, 1);
 }
@@ -374,7 +398,7 @@ set_icon_name (CcWacomPage *page,
 	priv = page->priv;
 
 	filename = g_strdup_printf ("%s.svg", icon_name);
-	path = g_build_filename (PIXMAP_DIR, filename, NULL);
+	path = g_build_filename (GNOMECC_UI_DIR, filename, NULL);
 	g_free (filename);
 
 	gtk_image_set_from_file (GTK_IMAGE (WID (widget_name)), path);
@@ -401,6 +425,10 @@ add_styli (CcWacomPage *page)
 		GtkWidget *page;
 
 		stylus = l->data;
+
+		if (gsd_wacom_stylus_get_stylus_type (stylus) == WACOM_STYLUS_TYPE_PUCK)
+			continue;
+
 		if (gsd_wacom_stylus_get_has_eraser (stylus)) {
 			GsdWacomDeviceType type;
 			type = gsd_wacom_stylus_get_stylus_type (stylus);
@@ -419,12 +447,73 @@ add_styli (CcWacomPage *page)
 	/*FIXME: Set the page with the last used item */
 }
 
+/* Different types of layout for the tablet config */
+enum {
+	LAYOUT_NORMAL,        /* tracking mode, button mapping */
+	LAYOUT_REVERSIBLE,    /* tracking mode, button mapping, left-hand orientation */
+	LAYOUT_SCREEN        /* button mapping, calibration, display resolution */
+};
+
+static void
+remove_left_handed (CcWacomPagePrivate *priv)
+{
+	gtk_widget_destroy (WID ("label-left-handed"));
+	gtk_widget_destroy (WID ("switch-left-handed"));
+}
+
+static void
+remove_display_link (CcWacomPagePrivate *priv)
+{
+	gtk_widget_destroy (WID ("display-link"));
+}
+
+static void
+update_tablet_ui (CcWacomPage *page,
+		  int          layout)
+{
+	CcWacomPagePrivate *priv;
+
+	priv = page->priv;
+
+	switch (layout) {
+	case LAYOUT_NORMAL:
+		remove_left_handed (page->priv);
+		remove_display_link (page->priv);
+		break;
+	case LAYOUT_REVERSIBLE:
+		remove_display_link (page->priv);
+		break;
+	case LAYOUT_SCREEN:
+		remove_left_handed (page->priv);
+
+		gtk_widget_destroy (WID ("combo-tabletmode"));
+		gtk_widget_destroy (WID ("label-trackingmode"));
+
+		gtk_widget_show (WID ("button-calibrate"));
+		gtk_widget_show (WID ("display-link"));
+
+		gtk_container_child_set (CWID ("main-grid"),
+					 WID ("tablet-buttons-box"),
+					 "left_attach", 1,
+					 "top_attach", 1, NULL);
+		gtk_container_child_set (CWID ("main-grid"),
+					 WID ("display-link"),
+					 "left_attach", 1,
+					 "top_attach", 2, NULL);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+}
+
 GtkWidget *
-cc_wacom_page_new (GsdWacomDevice *stylus,
+cc_wacom_page_new (CcWacomPanel   *panel,
+		   GsdWacomDevice *stylus,
 		   GsdWacomDevice *eraser)
 {
 	CcWacomPage *page;
 	CcWacomPagePrivate *priv;
+	int layout;
 
 	g_return_val_if_fail (GSD_IS_WACOM_DEVICE (stylus), NULL);
 	g_return_val_if_fail (gsd_wacom_device_get_device_type (stylus) == WACOM_TYPE_STYLUS, NULL);
@@ -435,6 +524,7 @@ cc_wacom_page_new (GsdWacomDevice *stylus,
 	page = g_object_new (CC_TYPE_WACOM_PAGE, NULL);
 
 	priv = page->priv;
+	priv->panel = panel;
 	priv->stylus = stylus;
 	priv->eraser = eraser;
 
@@ -445,18 +535,19 @@ cc_wacom_page_new (GsdWacomDevice *stylus,
 	/* Tablet name */
 	gtk_label_set_text (GTK_LABEL (WID ("label-tabletmodel")), gsd_wacom_device_get_name (stylus));
 
-	/* Left-handedness */
-	if (gsd_wacom_device_reversible (stylus) == FALSE) {
-		gtk_widget_hide (WID ("label-left-handed"));
-		gtk_widget_hide (WID ("switch-left-handed"));
-	} else {
-		set_left_handed_from_gsettings (page);
-	}
+	/* Type of layout */
+	if (gsd_wacom_device_is_screen_tablet (stylus))
+		layout = LAYOUT_SCREEN;
+	else if (gsd_wacom_device_reversible (stylus))
+		layout = LAYOUT_REVERSIBLE;
+	else
+		layout = LAYOUT_NORMAL;
 
-	/* Calibration for screen tablets */
-	if (gsd_wacom_device_is_screen_tablet (stylus) == TRUE) {
-		gtk_widget_show (WID ("button-calibrate"));
-	}
+	update_tablet_ui (page, layout);
+
+	/* Left-handedness */
+	if (gsd_wacom_device_reversible (stylus))
+		set_left_handed_from_gsettings (page);
 
 	/* Tablet icon */
 	set_icon_name (page, "image-tablet", gsd_wacom_device_get_icon_name (stylus));
