@@ -214,7 +214,7 @@ update_current_page (CcWacomPanel *self)
 
 		if (g_hash_table_lookup (priv->pages, tablet->name) == NULL) {
 			GtkWidget *page;
-			page = cc_wacom_page_new (tablet->stylus, tablet->eraser);
+			page = cc_wacom_page_new (self, tablet->stylus, tablet->eraser);
 			cc_wacom_page_set_navigation (CC_WACOM_PAGE (page), GTK_NOTEBOOK (priv->notebook), TRUE);
 			gtk_widget_show (page);
 			gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), page, NULL);
@@ -277,17 +277,24 @@ device_added_cb (GdkDeviceManager *manager,
 static gboolean
 link_activated (GtkLinkButton *button,
 		CcWacomPanel  *self)
+{
+	cc_wacom_panel_switch_to_panel (self, "bluetooth");
+	return TRUE;
+}
+
+void
+cc_wacom_panel_switch_to_panel (CcWacomPanel *self,
+				const char   *panel)
 {
 	CcShell *shell;
 	GError *error = NULL;
 
 	shell = cc_panel_get_shell (CC_PANEL (self));
-	if (cc_shell_set_active_panel_from_id (shell, "bluetooth", NULL, &error) == FALSE)
+	if (cc_shell_set_active_panel_from_id (shell, panel, NULL, &error) == FALSE)
 	{
-		g_warning ("Failed to activate Bluetooth panel: %s", error->message);
+		g_warning ("Failed to activate '%s' panel: %s", panel, error->message);
 		g_error_free (error);
 	}
-	return TRUE;
 }
 
 static void
@@ -337,7 +344,13 @@ cc_wacom_panel_init (CcWacomPanel *self)
 
 	gtk_notebook_set_show_tabs (notebook, FALSE);
 	gtk_widget_set_vexpand (GTK_WIDGET (notebook), TRUE);
-	gtk_container_set_border_width (GTK_CONTAINER (notebook), 24);
+	gtk_container_set_border_width (GTK_CONTAINER (notebook), 0);
+	g_object_set (G_OBJECT (notebook),
+		      "margin-top", 0,
+		      "margin-right", 24,
+		      "margin-left", 24,
+		      "margin-bottom", 24,
+		      NULL);
 
 	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (notebook));
 	gtk_widget_show (priv->notebook);
